package com.despark.tuning3d.view
{
import com.despark.tuning3d.ApplicationFacade;
import com.despark.tuning3d.Notifications;
import com.despark.tuning3d.model.core.Device;
import com.despark.tuning3d.model.friend.FriendProxy;
import com.despark.tuning3d.model.languages.Language;
import com.despark.tuning3d.model.languages.LanguagesProxy;
import com.despark.tuning3d.model.settings.Settings;
import com.despark.tuning3d.model.settings.SettingsProxy;
import com.despark.tuning3d.model.users.UsersProxy;
import com.despark.tuning3d.model.votes.VotesProxy;
import com.despark.tuning3d.view.components.brands.BrandsScreen;
import com.despark.tuning3d.view.components.core.AlertView;
	import com.despark.tuning3d.view.components.core.BannerStackedScreen;
	import com.despark.tuning3d.view.components.core.StackedScreen;
import com.despark.tuning3d.view.components.instructions.TutorialView;
import com.despark.tuning3d.view.components.votes.VotesListScreen;
import com.despark.tuning3d.view.core.StackedScreenMediator;
import com.utils.ArrayUtil;

import org.puremvc.as3.interfaces.INotification;

import starling.core.Starling;
import starling.events.Event;

public class VotesListScreenMediator extends StackedScreenMediator
	{
		//private var allTunningsLoaded : Boolean;
		//private static var b : int;
		
		private var screenShown:Boolean=false;
		//private var pendingForAuthAction : String;
		private var _garage_delay:Number=0.5;

		private static const AUTH_ACTION_GARAGE : String = "garage";
        public static const AUTH_ACTION_FRIENDS : String = "friends";
		
		
		
		public function VotesListScreenMediator(stackedScreen : StackedScreen)
		{
			super(stackedScreen);
		}

		override public function onRegister() : void
		{
			super.onRegister();

		//	votesListScreen.addEventListener(VotesListScreen.PAGE_REACHED, votesScreenPageReachedHandler);

			votesListScreen.addEventListener(Event.ADDED_TO_STAGE, votesScreenAddedToStageHandler);

			votesListScreen.addEventListener(StackedScreen.VOTE_SELECTED, votesScreenVoteSelectedHandler);
			votesListScreen.addEventListener(VotesListScreen.CHANGE_SEARCH_TEXT, votesScreenChangeSearchTextHandler);
			votesListScreen.addEventListener(VotesListScreen.VOTE_TYPE_CHANGE, votesScreenTypeChangeHandler);

			votesListScreen.addEventListener(StackedScreen.GARAGE_SELECTED, votesScreenGarageSelectedHandler);
			votesListScreen.addEventListener(BrandsScreen.LANGUAGE_CHANGED,votesScreenLanguageChangedHandler);
			votesListScreen.addEventListener(BrandsScreen.LOGOUT_SELECTED, votesScreenLogoutSelectedHandler);
			votesListScreen.addEventListener(BrandsScreen.LOGIN_SELECTED, loginSelectedHandler);
			votesListScreen.addEventListener(StackedScreen.BRANDS_SELECTED, brandsSelectedHandler);
			votesListScreen.addEventListener(StackedScreen.HOME_SELECTED, votesScreenHomeSelectedSelectedHandler);
			votesListScreen.addEventListener(BrandsScreen.INSTRUCTION_SELECTED, votesScreenInstructionSelectedHandler);

            votesListScreen.addEventListener(StackedScreen.TIMELINE_SECLECTED, votesScreenTimelineSelectedHandler);
            votesListScreen.addEventListener(StackedScreen.GALLERY_SELECTED, gallerySelectHandler);
            votesListScreen.addEventListener(StackedScreen.FRIENDS_SELECTED, votesScreenFriendsSelectedHandler);
            votesListScreen.addEventListener(StackedScreen.NOTIFICATION_SELECTED, votesScreenNotificationsSelectedHandler);
            votesListScreen.addEventListener(StackedScreen.CHALLENGE_NOTIFICATIONS_SELECTED, challengeNotesSelectHandler);
			votesListScreen.addEventListener(StackedScreen.CHALLENGES_SELECTED, challengeSelectHandler);
            votesListScreen.addEventListener(StackedScreen.LEADER_BOARD_SELECTED, leaderBoardSelectHandler);
		}

		private function challengeSelectHandler(event : Event): void {
			sendNotification(Notifications.DISPLAY_CHALLENGE_SCREEN);
		}

        private function leaderBoardSelectHandler(event : Event) : void {
            sendNotification(Notifications.DISPLAY_CHALLENGE_LEADERBOARD_POPUP, {deleteOtherScreens : false, enableCloseButton: true});
        }

        private function gallerySelectHandler(event : Event) : void {
            sendNotification(Notifications.DISPLAY_GALLERY_SCREEN);
        }

		private function handleUserAuthenticated() : void {
			if (votesListScreen){
				votesListScreen.userSignedIn = true;
				votesListScreen.mainMenuView.loginButtonTriggeredHandler();
				if (votesListScreen.mainMenuView.pendingForAuthAction == BrandsScreenMediator.AUTH_ACTION_LOGIN) {
					return;
				}
			}

			if (stackedScreen && !stackedScreen.stage) {

				return;
			}

			if (pendingForAuthAction == BrandsScreenMediator.AUTH_ACTION_GARAGE)
			{


				Starling.juggler.delayCall(function() : void{

					sendNotification(Notifications.DISPLAY_TUNINGS_LIST_SCREEN);

				}, _garage_delay);
			}
			else  if(pendingForAuthAction == BrandsScreenMediator.AUTH_ACTION_TIMELINE)
            {
                Starling.juggler.delayCall(function() : void{

                    sendNotification(Notifications.DISPLAY_TIMELINE_SCREEN);

                }, _garage_delay)
            }

            if (pendingForAuthAction == AUTH_ACTION_FRIENDS){
                Starling.juggler.delayCall(function() : void{
					BannerStackedScreen.hideAdBanner(false);
                    sendNotification(Notifications.DISPLAY_FRIENDS_LIST_SCREEN, {showBannerAfterExit: true});
                }, _garage_delay);
            }

            if (pendingForAuthAction == BrandsScreenMediator.AUTH_ACTION_NOTIFICATIONS){
                Starling.juggler.delayCall(function() : void{
					BannerStackedScreen.hideAdBanner(false);
                    sendNotification(Notifications.DISPLAY_IN_APP_NOTIFICATION_POPUP, {showBannerAfterExit: true});
                }, 0.5);
            } else if (pendingForAuthAction == BrandsScreenMediator.AUTH_ACTION_CHALLENGE_NOTIFICATIONS){
                Starling.juggler.delayCall(function() : void{
					BannerStackedScreen.hideAdBanner(false);
                    sendNotification(Notifications.DISPLAY_CHALLENGE_NOTIFICATION_POPUP, {showBannerAfterExit: true});
                }, 0.5);
            }
            var friendProxy:FriendProxy = ApplicationFacade.getInstance().retrieveProxy(FriendProxy.NAME) as FriendProxy;
            friendProxy.loadPendings();
		}

        private function votesScreenNotificationsSelectedHandler(event:Event):void
        {
            pendingForAuthAction = BrandsScreenMediator.AUTH_ACTION_NOTIFICATIONS;
            sendNotification(Notifications.AUTHENTICATE_USER);
        }

        private function challengeNotesSelectHandler(event : Event) : void {
            pendingForAuthAction = BrandsScreenMediator.AUTH_ACTION_CHALLENGE_NOTIFICATIONS;
            sendNotification(Notifications.AUTHENTICATE_USER);
        }


        private function votesScreenFriendsSelectedHandler(event:Event):void
        {
            var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;

            if(!usersProxy.currentUser) _garage_delay=1.5;
            else _garage_delay=0.2;

            pendingForAuthAction = AUTH_ACTION_FRIENDS;
            sendNotification(Notifications.AUTHENTICATE_USER);
        }


        private function votesScreenTimelineSelectedHandler(event:Event):void{

            var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;

            if(!usersProxy.currentUser) _garage_delay=1.5;
            else _garage_delay=0.2;


            pendingForAuthAction = BrandsScreenMediator.AUTH_ACTION_TIMELINE;;
            sendNotification(Notifications.AUTHENTICATE_USER);
        }

		private function votesScreenInstructionSelectedHandler(event:Event):void{
			TutorialView.showCustomUrl();
		}

		private function brandsSelectedHandler(event:Event):void {
			if(HomeScreenPopUpMediator.isFirstOpen) {
				sendNotification(Notifications.DISPLAY_BRANDS_SCREEN);
			} else {
				sendNotification(Notifications.DISPLAY_ROOT_SCREEN);
			}
			HomeScreenPopUpMediator.isFirstOpen = false;
		}

		private function votesScreenHomeSelectedSelectedHandler(event:Event):void{

			sendNotification(Notifications.DISPLAY_HOME_SCREEN_POPUP);
		}



		private function votesScreenGarageSelectedHandler(event:Event):void{

			var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;

			if(!usersProxy.currentUser) _garage_delay=1.5;
			else _garage_delay=0.2;


			pendingForAuthAction = BrandsScreenMediator.AUTH_ACTION_GARAGE;
			sendNotification(Notifications.AUTHENTICATE_USER);
		}

		override public function onRemove() : void
		{
			super.onRemove();

			//votesListScreen.removeEventListener(VotesListScreen.PAGE_REACHED, votesScreenPageReachedHandler);
			votesListScreen.removeEventListener(Event.ADDED_TO_STAGE, votesScreenAddedToStageHandler);
			votesListScreen.removeEventListener(StackedScreen.VOTE_SELECTED, votesScreenVoteSelectedHandler);
			votesListScreen.removeEventListener(VotesListScreen.CHANGE_SEARCH_TEXT, votesScreenChangeSearchTextHandler);
			votesListScreen.removeEventListener(VotesListScreen.VOTE_TYPE_CHANGE, votesScreenTypeChangeHandler);
			votesListScreen.removeEventListener(StackedScreen.GARAGE_SELECTED, votesScreenGarageSelectedHandler);
			votesListScreen.removeEventListener(BrandsScreen.LANGUAGE_CHANGED,votesScreenLanguageChangedHandler);
			votesListScreen.removeEventListener(BrandsScreen.LOGOUT_SELECTED, votesScreenLogoutSelectedHandler);
			votesListScreen.removeEventListener(BrandsScreen.LOGIN_SELECTED, loginSelectedHandler);
			votesListScreen.removeEventListener(StackedScreen.BRANDS_SELECTED, brandsSelectedHandler);
			votesListScreen.removeEventListener(StackedScreen.HOME_SELECTED, votesScreenHomeSelectedSelectedHandler);
			votesListScreen.removeEventListener(BrandsScreen.INSTRUCTION_SELECTED, votesScreenInstructionSelectedHandler);

            votesListScreen.removeEventListener(StackedScreen.TIMELINE_SECLECTED, votesScreenTimelineSelectedHandler);
            votesListScreen.removeEventListener(StackedScreen.GALLERY_SELECTED, gallerySelectHandler);
            votesListScreen.removeEventListener(StackedScreen.FRIENDS_SELECTED, votesScreenFriendsSelectedHandler);
            votesListScreen.removeEventListener(StackedScreen.NOTIFICATION_SELECTED, votesScreenNotificationsSelectedHandler);
            votesListScreen.removeEventListener(StackedScreen.CHALLENGE_NOTIFICATIONS_SELECTED, challengeNotesSelectHandler);
			votesListScreen.removeEventListener(StackedScreen.CHALLENGES_SELECTED, challengeSelectHandler);

			votesListScreen.dispose();
		}

		public function get votesListScreen() : VotesListScreen
		{
			return viewComponent as  VotesListScreen;
		}
		
		override public function listNotificationInterests() : Array
		{
            return ArrayUtil.uniqueConcat(super.listNotificationInterests(), [
				Notifications.USER_AUTHENTICATED,
				Notifications.VOTES_LOADED,
				Notifications.VOTES_LOADING_FAILED,
				Notifications.APPLICATION_BECAME_ACTIVE,
				Notifications.LANGUAGES_LOADED,
				Notifications.LANGUAGES_LOADING_FAILED,
				Notifications.LANGUAGE_LOADED,
				Notifications.LANGUAGE_LOADING_FAILED,
				Notifications.USER_LOGGED_OUT,
				Notifications.USER_AUTHENTICATED,
				Notifications.USER_SESSION_EXTENDED,
				Notifications.USER_SESSION_EXTENDING_FAILED,
				Notifications.USER_SESSION_EXTENDING_NOT_POSSIBLE,
                Notifications.TIMELINE_SCREEN_FEED_PREVIEW
			]);
		}
		
		override public function handleNotification(note : INotification) : void 
		{
			switch(note.getName()) 
			{				
				case Notifications.VOTES_LOADED:
					handleVotesLoaded(note.getBody());
					break;
				
				case Notifications.VOTES_LOADING_FAILED:
					handleVotesLoadingFailed();
					break;
				
			
				case Notifications.APPLICATION_BECAME_ACTIVE:{
					
					handleApplicationBecameActive();
				}
					break;
				
				case Notifications.LANGUAGES_LOADED:
					handleLanguagesLoaded();
					break;
				
				case Notifications.LANGUAGES_LOADING_FAILED:
					handleLanguagesLoadingFailed();
					break;
				
				case Notifications.LANGUAGE_LOADED:
					handleLanguageLoaded(note.getBody().language);
					break;
				
				case Notifications.LANGUAGE_LOADING_FAILED:
					handleLanguageLoadingFailed();
					break;
				
				case Notifications.USER_LOGGED_OUT:
					handleUserLoggedOut();
					break;
				
				case Notifications.USER_AUTHENTICATED:
					handleUserAuthenticated();
					break;
				
				case Notifications.USER_SESSION_EXTENDED:
					handleUserSessionExtented();
					break;
				
				case Notifications.USER_SESSION_EXTENDING_FAILED:
					handleUserSessionExtendingFailed();
					break;
				
				case Notifications.USER_SESSION_EXTENDING_NOT_POSSIBLE:
					handleUserSessionExtendingNotPossible();
					break;

                case Notifications.TIMELINE_SCREEN_FEED_PREVIEW:
                    handleDisplayTimelineScreenFeed(note.getBody());
				
			}
			
			super.handleNotification(note);
		}
		
		private function handleUserSessionExtendingNotPossible() : void {
			/*	var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
			usersProxy.logoutCurrentUser();
			sendNotification(Notifications.DISPLAY_LOGIN_SCREEN);*/
		}
		
		private function handleUserSessionExtented() : void {
			
		}
		
		private function handleUserSessionExtendingFailed() : void {
			
		}

        private function handleDisplayTimelineScreenFeed(data:Object):void {
            sendNotification(Notifications.DISPLAY_TIMELINE_SCREEN, data);
        }
		
		private function handleUserLoggedOut() : void
		{
			votesListScreen.userSignedIn = false;
            sendNotification(Notifications.PENDING_FRIENDS_LOADED, {friends: null, unread:false});
		}
		
		private function handleLanguageLoadingFailed() : void
		{
			
		}
		
		private function handleLanguagesLoadingFailed() : void
		{
			votesListScreen.selectedLanguage = null;
		}
		
		private function handleLanguageLoaded(loadedLanguage : Language) : void
		{
			if(votesListScreen)
			{
				votesListScreen.updateWithLanguage(loadedLanguage);
				
				updateSettings();
				
				sendNotification(Notifications.LANGUAGE_CHANGED);
			}
		}
		
		private function handleLanguagesLoaded():void{
			
			
			Starling.juggler.delayCall(refreshSettings, 1.5);
		}
		
		
		private function votesScreenLanguageChangedHandler(event:Event):void{
			
			var languagesProxy : LanguagesProxy = facade.retrieveProxy(LanguagesProxy.NAME) as LanguagesProxy;
			languagesProxy.loadLanguage(votesListScreen.selectedLanguage);
			
			
		}
		
		
		private function updateSettings() : void
		{
			var settingsProxy : SettingsProxy = facade.retrieveProxy(SettingsProxy.NAME) as SettingsProxy;
			var settings : Settings = settingsProxy.currentSettings;
			
			//settings.autoscrollEnabled = brandsScreen.autoscrollEnabled;
			
			if(votesListScreen.selectedLanguage)
			{
				settings.selectedLanguageCode = votesListScreen.selectedLanguage.code;
			}
			
			settingsProxy.update(settings);
		}
		
		
		
		private function votesScreenLogoutSelectedHandler(event:Event) : void
		{
			var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
			usersProxy.logoutCurrentUser();
			
			AlertView.show(AlertView.language.translate("mob_saving", "loggedOut"), -1, false);   
			

			this.votesListScreen.updateNotifications();
			this.votesListScreen.ShowDone();
		   //	sendNotification(Notifications.DISPLAY_ROOT_SCREEN);
		}

		private function loginSelectedHandler(event : Event) : void {
			pendingForAuthAction = BrandsScreenMediator.AUTH_ACTION_LOGIN;
			sendNotification(Notifications.AUTHENTICATE_USER);
		}
		
		private function refreshSettings() : void {
			var settingsProxy : SettingsProxy = facade.retrieveProxy(SettingsProxy.NAME) as SettingsProxy;
			var languagesProxy : LanguagesProxy = facade.retrieveProxy(LanguagesProxy.NAME) as LanguagesProxy;
			var settings : Settings = settingsProxy.currentSettings;
			
			
			if(!settings.selectedLanguageCode)
			{
				settings.selectedLanguageCode = languagesProxy.defaultLanguageCode;
			}
			
			//this.brandsScreen
			
			votesListScreen.selectedLanguage = languagesProxy.getLanguage(settings.selectedLanguageCode);
			
			

			if(!votesListScreen.selectedLanguage){
				
				settings.selectedLanguageCode="en-US";
				votesListScreen.selectedLanguage = languagesProxy.getLanguage(settings.selectedLanguageCode);
				
			}
			//brandsScreen.settings = settings;
			

			
			votesListScreen.languages = languagesProxy.languages;
			
			if(votesListScreen.languages!=null && votesListScreen.languages.length<=0){
				
				//languagesProxy  = facade.retrieveProxy(LanguagesProxy.NAME) as LanguagesProxy;
				languagesProxy.loadLanguages();
			}else if (votesListScreen.languages==null){
				
				//languagesProxy  = facade.retrieveProxy(LanguagesProxy.NAME) as LanguagesProxy;
				languagesProxy.loadLanguages();
			}
			
		}
		
		
			
		private function handleApplicationBecameActive() : void
		{
			if(Device.isIos())
				return;
			
			
			if(this.votesListScreen){
				
				votesListScreen.invalidate();
			}
		}
		
		
		protected override function stackedScreenBackButtonSelectedHandler(event : Event) : void
		{
			super.stackedScreenBackButtonSelectedHandler(event);
		}
		
		/*private function votesScreenPageReachedHandler(event : Event) : void 
		{
			var isLastPageReached : Boolean = votesListScreen.pageReached == votesListScreen.pagesCount - 1;
			
			if(isLastPageReached )
			{	
				if(!allTunningsLoaded)
				{
					fetchNextVotes();
				}
			}
		}*/
		
		private function handleVotesLoaded(votesInfo : Object) : void
		{
			
			/*for each(var vote:Vote in tunningsInfo.votes){
				
				trace("tuning.car="+tuning.car);
				
				if(tuning.car){
					
					trace("tuning.car.id:"+tuning.car.id);
				}
			}*/
			
			votesListScreen.votes=votesInfo.votes;
			
			//allTunningsLoaded = tunningsInfo.lastPage;
			
			if(alertView)
			{
				alertView.hide();
			}
			
			//votesListScreen.showNoTuningsMessage(votesListScreen.pageReached == 0 && tunningsInfo.tunnings.length == 0 && votesListScreen.loadedTuningsCount == 0);
			
			
			//addAds();  
			
		}
		
		private function handleVotesLoadingFailed() : void
		{
			if(alertView)
			alertView.hide();
		}
		
		/*private function fetchNextVotes() : void
		{
			var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
			var tuningsProxy : TuningsProxy = facade.retrieveProxy(TuningsProxy.NAME) as TuningsProxy;
			
			tuningsProxy.loadTunings(votesListScreen.loadedTuningsCount, usersProxy.currentUser);
			
			if(alertView)
			{
				alertView.hide();
			}
				
			alertView = AlertView.show(AlertView.language.translate("mob_alertsAndOverlays", "loading"), -1);
		}*/


		
		
		private function votesScreenAddedToStageHandler(event : Event=null) : void 
		{			
			if(!screenShown)
			{			
				screenShown = true;
				
				var languagesProxy : LanguagesProxy = facade.retrieveProxy(LanguagesProxy.NAME) as LanguagesProxy;
				languagesProxy.loadLanguages();
				
			
				var settingsProxy : SettingsProxy = facade.retrieveProxy(SettingsProxy.NAME) as SettingsProxy;
				var languageToLoad : Language = new Language(null);
				languageToLoad.code = settingsProxy.currentSettings.selectedLanguageCode;
				languagesProxy.loadLanguage(languageToLoad);
				
				
				
				var votesProxy :VotesProxy = facade.retrieveProxy(VotesProxy.NAME) as VotesProxy;
				votesProxy.load(votesListScreen.queryText, votesListScreen.votingType);

                Starling.juggler.delayCall(checkIfVotesLoadedDownloaded, 0.3);
			}
			
		}

		private function checkIfVotesLoadedDownloaded() : void
		{
			if(votesListScreen.loadedVotesCount == 0)
			{
                var votesProxy :VotesProxy = facade.retrieveProxy(VotesProxy.NAME) as VotesProxy;
				votesProxy.load(votesListScreen.queryText, votesListScreen.votingType);

                Starling.juggler.delayCall(checkIfVotesLoadedDownloaded, 0.2);
			}

		}
		
		private function votesScreenChangeSearchTextHandler(event:Event):void{
			
			//trace("votesScreenChangeSearchTextHandler:", event.data.text);
			
			var votesProxy :VotesProxy = facade.retrieveProxy(VotesProxy.NAME) as VotesProxy;
			votesProxy.load(votesListScreen.queryText, votesListScreen.votingType);
		}

		private function votesScreenTypeChangeHandler(event:Event):void{
			var votesProxy :VotesProxy = facade.retrieveProxy(VotesProxy.NAME) as VotesProxy;
			votesProxy.load(votesListScreen.queryText, votesListScreen.votingType);
		}


		private function votesScreenVoteSelectedHandler(event : Event) : void
		{

			var votesProxy :VotesProxy = facade.retrieveProxy(VotesProxy.NAME) as VotesProxy;
			if(votesListScreen.selectedVote){
				
				if(votesListScreen.selectedVote.youHaveLiked){
					votesProxy.vote(votesListScreen.selectedVote.id);
			
				}else{
					
					//trace("delete vote for:", votesListScreen.selectedVote.id);
					votesProxy.delete_vote(votesListScreen.selectedVote.id);
					
					
				}
			}
		}
		

	}
}