package com.despark.tuning3d.view.components.core {
import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.core.UIColors;
    import com.despark.tuning3d.view.components.timeline.components.TimelineLabelFactory;
import com.utils.FontSelector;

import feathers.controls.Label;

import feathers.controls.text.StageTextTextEditor;
import feathers.controls.text.TextFieldTextRenderer;
import feathers.core.FeathersControl;
import feathers.core.IMultilineTextEditor;
import feathers.core.ITextRenderer;

import flash.text.TextFormat;
import flash.text.TextFormatAlign;
import flash.text.engine.FontPosture;
import flash.text.engine.FontWeight;

// ITextBaselineControl, IFeathersControl
public class LabelFactory {
    private static const TEXT_COLOR_BLACK:uint = 0x242424;
    private static const TEXT_COLOR_FRIEND_LIST:uint = 0x58595B;
    //
    private static const FONT_SIZE_FRIEND_LIST:uint = 10;
    public function LabelFactory() {
    }

    public static function  garageSettingLabel ():ITextRenderer {
        var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
        textRenderer.textFormat = new TextFormat(FontSelector.getMonserratMedium(), 14, UIColors.WHITE);
        textRenderer.textFormat.italic = false;
        textRenderer.embedFonts = true;
        textRenderer.wordWrap = true;
        textRenderer.textFormat.align = TextFormatAlign.RIGHT;
        return textRenderer;
    }

    public static function  challengesStartChallengeLabel ():ITextRenderer {
        var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
        textRenderer.textFormat = new TextFormat(FontSelector.getMonserratMedium(), 11, UIColors.WHITE);
        textRenderer.textFormat.italic = false;
        textRenderer.embedFonts = true;
        textRenderer.wordWrap = true;
        textRenderer.textFormat.align = TextFormatAlign.RIGHT;
        return textRenderer;
    }

    public static function  notificationsMessageField ():ITextRenderer {
        var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
        textRenderer.textFormat = new TextFormat(FontSelector.getMonserratMedium(), 12, UIColors.UI_COMPONENT_GRAY);
        textRenderer.textFormat.italic = false;
        textRenderer.embedFonts = true;
        textRenderer.wordWrap = true;
        textRenderer.textFormat.align = TextFormatAlign.CENTER;
        return textRenderer;
    }

    public static function createLabelPopup():FeathersControl{
        var result:FeathersControl;
        if(Constants.isIOS) {
            result = LabelFactory.usernameLabelPopupIos() as FeathersControl;
        }else{
            result = new Label();
            Label(result).textRendererFactory = LabelFactory.nameLabelRendererPopupAndroid;
            Label(result).wordWrap = false;
        }
        return result;
    }

    public static function createLabelLarge():FeathersControl{
        var result:FeathersControl;
        if(Constants.isIOS) {
            result = LabelFactory.usernameLabelLargeIos() as FeathersControl;
        }else{
            result = new Label();
            Label(result).textRendererFactory = LabelFactory.nameLabelRendererLargeAndroid;
            Label(result).wordWrap = false;
        }
        return result;
    }

    public static function createLabelTimelineCarName():FeathersControl{
        var result:FeathersControl;
        if(Constants.isIOS) {
            result = LabelFactory.carNameTimelineLabelLargeIos() as FeathersControl;
        }else{
            result = new Label();
            Label(result).textRendererFactory = LabelFactory.carNameLabelRendererLargeAndroid;
            Label(result).wordWrap = false;
        }
        return result;
    }

    public static function createLabelComment():FeathersControl{
        var result:FeathersControl;
        if(Constants.isIOS) {
            result = LabelFactory.usernameLabelCommentIos() as FeathersControl;
        }else{
            result = new Label();
            Label(result).textRendererFactory = LabelFactory.nameLabelRendererCommentAndroid;
            Label(result).wordWrap = false;
        }
        return result;
    }

    public static function createLabelFriendList():FeathersControl{
        var result:FeathersControl;
        if(Constants.isIOS) {
            result = LabelFactory.nameLabelFriendListIos() as FeathersControl;
        }else{
            result = new Label();
            Label(result).textRendererFactory = LabelFactory.nameLabelFriendListAndroid;
            Label(result).wordWrap = false;
        }
        return result;
    }

    public static function createLabelTimelineDesc():FeathersControl{
        var result:FeathersControl;
        if(Constants.isIOS) {
            result = LabelFactory.labelTimelineDescIos() as FeathersControl;
        }else{
            result = new Label();
            Label(result).textRendererFactory = LabelFactory.labelTimelineDescAndroid;
            Label(result).wordWrap = false;
        }
        return result;
    }

    // ************** Android ******************
    public static function  nameLabelRendererCommentAndroid ():ITextRenderer{
        return getAndroidRenderer(TimelineLabelFactory.SIZE_BIG, TimelineLabelFactory.COLOR_TEXT_GREY, true, false, TextFormatAlign.LEFT);
    }

    public static function  nameLabelRendererLargeAndroid ():ITextRenderer{
        return getAndroidRenderer(TimelineLabelFactory.SIZE_BIG_NAME, TimelineLabelFactory.COLOR_TEXT_GREY, true, false, TextFormatAlign.LEFT);
    }

    public static function  carNameLabelRendererLargeAndroid ():ITextRenderer{
        return getAndroidRenderer(TimelineLabelFactory.SIZE_BIG_NAME, TimelineLabelFactory.COLOR_TEXT_RED, true, false, TextFormatAlign.LEFT, false);
    }

    private static function nameLabelRendererPopupAndroid() :ITextRenderer {
        return getAndroidRenderer(TimelineLabelFactory.SIZE_BIG_NAME, TEXT_COLOR_BLACK, false, true, TextFormatAlign.LEFT);
    }

    private static function nameLabelFriendListAndroid() :ITextRenderer {
        return getAndroidRenderer(FONT_SIZE_FRIEND_LIST, TEXT_COLOR_FRIEND_LIST, true, false, TextFormatAlign.LEFT);
    }

    private static function labelTimelineDescAndroid() :ITextRenderer {
        return getAndroidRenderer(TimelineLabelFactory.SIZE_BIG, TimelineLabelFactory.COLOR_TEXT_GREY, false, false, TextFormatAlign.LEFT);
    }

    private static function getAndroidRenderer(fontSize:int, textColor:uint, bold:Boolean = false, italic:Boolean = false, align:String = null, hasFilter:Boolean = true):ITextRenderer{
        align ||= TextFormatAlign.LEFT;
        var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
        textRenderer.textFormat = new TextFormat(FontSelector.getUnicode(), fontSize, textColor);
        textRenderer.textFormat.italic = italic;
        textRenderer.textFormat.bold = bold;
        textRenderer.textFormat.align = align;
        textRenderer.filter = hasFilter?TimelineLabelFactory.getFilter():null;
        return textRenderer;
    }

    // ************** IOS **********************
    public static function  usernameLabelCommentIos ():IMultilineTextEditor {
        return getIOSRenderer(TimelineLabelFactory.SIZE_BIG, TimelineLabelFactory.COLOR_TEXT_GREY, true, false, TextFormatAlign.LEFT);
    }

    public static function  usernameLabelLargeIos ():IMultilineTextEditor {
        return getIOSRenderer(TimelineLabelFactory.SIZE_BIG_NAME, TimelineLabelFactory.COLOR_TEXT_GREY, true, false, TextFormatAlign.LEFT);
    }

    public static function  carNameTimelineLabelLargeIos ():IMultilineTextEditor {
        return getIOSRenderer(TimelineLabelFactory.SIZE_BIG_NAME, TimelineLabelFactory.COLOR_TEXT_RED, true, false, TextFormatAlign.LEFT);
    }

    public static function  usernameLabelPopupIos ():IMultilineTextEditor {
        return getIOSRenderer(TimelineLabelFactory.SIZE_BIG_NAME, TEXT_COLOR_BLACK, false, true, TextFormatAlign.LEFT);
    }

    public static function  nameLabelFriendListIos ():IMultilineTextEditor {
        return getIOSRenderer(FONT_SIZE_FRIEND_LIST, TEXT_COLOR_FRIEND_LIST, true, false, TextFormatAlign.LEFT);
    }

    public static function  labelTimelineDescIos ():IMultilineTextEditor {
        return getIOSRenderer(TimelineLabelFactory.SIZE_BIG, TimelineLabelFactory.COLOR_TEXT_GREY, false, false, TextFormatAlign.LEFT);
    }

    private static function getIOSRenderer(fontSize:int, textColor:uint, bold:Boolean = false, italic:Boolean = false, align:String = null):IMultilineTextEditor{
        align ||= TextFormatAlign.LEFT;
        var textEditor:StageTextTextEditor = new StageTextTextEditor();
        textEditor.fontSize = fontSize;
        textEditor.color = textColor;
        textEditor.multiline = false;
        textEditor.isEditable = false;
        if(bold){
            textEditor.fontWeight = FontWeight.BOLD;
        }
        if(italic){
            textEditor.fontPosture = FontPosture.ITALIC;
        }
        textEditor.textAlign = align;
        textEditor.fontFamily = "Arial";
        return textEditor;
    }

}
}
