package com.despark.tuning3d.view.components.friends
{
import com.despark.tuning3d.model.core.Constants;
import com.despark.tuning3d.model.friend.Friend;
import com.despark.tuning3d.model.friend.IFriendHolder;
import com.despark.tuning3d.view.components.NotificationView.NotificationPanelLabelFactory;
import com.despark.tuning3d.view.components.core.ButtonFactory;
import com.despark.tuning3d.view.components.core.LabelFactory;
import com.despark.tuning3d.view.components.core.ListItemRenderer;
import com.despark.tuning3d.view.components.core.MultilingualButton;
import com.despark.tuning3d.view.components.core.MultilingualLabel;
import com.despark.tuning3d.view.components.core.SlidingButton;
import com.despark.tuning3d.view.components.core.StackedScreen;
import com.despark.tuning3d.view.components.timeline.components.UserCircularAvatar;
import com.utils.DateParser;
import com.utils.FontSelector;

import feathers.controls.Label;

import feathers.controls.LayoutGroup;

import feathers.controls.text.TextFieldTextRenderer;
import feathers.core.FeathersControl;
import feathers.core.ITextRenderer;
import feathers.layout.AnchorLayout;
import feathers.layout.AnchorLayoutData;
import feathers.layout.AnchorLayoutData;

import flash.text.TextFormat;
import flash.text.TextFormatAlign;

import starling.display.DisplayObject;
import starling.display.Quad;
import starling.display.Sprite;
import starling.events.Event;
import starling.events.Touch;
import starling.events.TouchEvent;
import starling.events.TouchPhase;

public class FriendItemRenderer extends ListItemRenderer implements IFriendHolder
	{
		private var verSpacer    : Quad;
        private var _userAvatar : UserCircularAvatar;

        private var _gap : int = 8;
		private var _avatarSize : Number = 40;
        private var _avatarXpos : Number =  8;
        private var _lastFriendData : Friend;

        protected var _nameLabel      : FeathersControl;
        protected var _requestLabel   : MultilingualLabel;
        protected var _dateLabel:Label;
        protected var _descriptionalLabel:Label;

        private var _undoFriendButton : MultilingualButton;
        private var _addFriendButton  : MultilingualButton;
        private var _approveButton    : MultilingualButton;
        private var _rejectButton     : MultilingualButton;
        private var _unfriendButton   : MultilingualButton;
        private var _slidedButton     : SlidingButton;

        private var _avatarContainer : LayoutGroup;
        private var _buttonContainer : LayoutGroup;
        private var _mainContainer   : LayoutGroup;

        private var _itemsTohide : Array = [];

        private var _eventDispatcher : *;
        public function set eventDispatcher(value:*) : void{
            _eventDispatcher = value;
        }
        public function get eventDispatcher() : *{
            return _eventDispatcher;
        }

        public function get friend() : Friend {
			return data as Friend;
		}

		protected override function initialize() : void
		{
			super.initialize();

            this._mainContainer = new LayoutGroup();
            this._mainContainer.layout = new AnchorLayout();
            this._mainContainer.width = actualWidth;
            this._mainContainer.height = actualHeight;
            addChild(this._mainContainer);

            var layData : AnchorLayoutData;

            /**
             * Avatar container
             */
            layData= new AnchorLayoutData();
            layData.verticalCenter = 0;
            layData.left = _avatarXpos;

            _avatarContainer = new LayoutGroup();
            _avatarContainer.layoutData = layData;
            this._mainContainer.addChild(_avatarContainer);
            _avatarContainer.addEventListener(TouchEvent.TOUCH, onFriendSelectHandler);

            /**
             * Name label
             */
            layData = new AnchorLayoutData();
            layData.verticalCenter = 0;
            layData.left = _gap;
            layData.leftAnchorDisplayObject = _avatarContainer;
            _nameLabel = LabelFactory.createLabelFriendList();
            _nameLabel.layoutData = layData;
            _mainContainer.addChild(_nameLabel);
            _nameLabel.addEventListener(TouchEvent.TOUCH, onFriendSelectHandler);


            /**
             * buttonsCotnainer
             */
            layData = new AnchorLayoutData();
            layData.verticalCenter = 0;
            layData.right = 15;
            this._buttonContainer = new LayoutGroup();
            this._buttonContainer.layout = new AnchorLayout();
            this._buttonContainer.layoutData = layData;
            this._mainContainer.addChild(_buttonContainer);

			verSpacer = new Quad(actualWidth - 5,.5,0xC1C3C2);
            verSpacer.alpha = .5;
			addChild(verSpacer);
		}

        private function descriptionTextRenderer():ITextRenderer
        {
            var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getFont(), 10, 0x58595B);
            textRenderer.textFormat.align = TextFormatAlign.LEFT;
            textRenderer.textFormat.italic = false;
            textRenderer.embedFonts = true;
            textRenderer.textFormat.bold = false;
            return textRenderer;
        }

        private function nameTextRenderer():ITextRenderer {
            var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getFont(), 10, 0x58595B);
            textRenderer.textFormat.italic = false;
            textRenderer.textFormat.bold = true;
            textRenderer.textFormat.align = TextFormatAlign.LEFT;
            return textRenderer;
        }

        private function detailsTextRenderer():ITextRenderer
        {
            var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
				textRenderer.textFormat = new TextFormat(FontSelector.getFont(), 7, 0x939393);
				textRenderer.textFormat.align = TextFormatAlign.LEFT;
				textRenderer.textFormat.italic = true;
				textRenderer.embedFonts = true;
				textRenderer.textFormat.bold = false;
            return textRenderer;
        }


		protected override function draw() : void
		{
			super.draw();

            this._mainContainer.width = actualWidth;
            this._mainContainer.height = actualHeight;
            this.verSpacer.y = actualHeight - 0.5;
            this.verSpacer.width = actualWidth-5;
		}


        private function hideAll() : void{
            for each (var item : DisplayObject in _itemsTohide){
                item.visible = false;
            }
        }

        private function buttonLayData() : AnchorLayoutData {
            var layData : AnchorLayoutData = new AnchorLayoutData();
            layData.verticalCenter = 0;
            layData.right = 0;
            return layData;
        }

		private function updateItemsVisibility() : void
		{
            hideAll();
            if(friend){
                _nameLabel['text'] = friend.user.nameToShow;

                if(!_userAvatar)
                {
                    _userAvatar = new UserCircularAvatar(friend.user, _avatarSize);
                    _avatarContainer.addChildAt(_userAvatar,0);
                }else {
                    _userAvatar.user = friend.user;
                    _userAvatar.visible = true;
                }

//                var arr: Array = [Friend.STATE_NO_FRIEND, Friend.STATE_REQUEST_SENT,Friend.STATE_PENDING, Friend.STATE_FRIEND]
//
//                var randState : int = Math.random() * 4;
//                var state : String = arr[randState];

                switch (friend.state){
                    case Friend.STATE_NO_FRIEND:
                        if (!_addFriendButton) {
                            _addFriendButton = ButtonFactory.newButton('add_friend',"add friend",0xFFFFFF,12);
                            _addFriendButton.layoutData = buttonLayData();
                            _addFriendButton.addEventListener(Event.TRIGGERED,sendFriendRequestEventHandler);
                            _buttonContainer.addChild(_addFriendButton);
                            _itemsTohide.push(_addFriendButton);
                        }else {
                            _addFriendButton.visible = true;
                        }
                        break;
                    case Friend.STATE_FRIEND:
                        if (!_slidedButton)
                        {
                            _unfriendButton = ButtonFactory.newButton("swipe_remove","",0xC1C3C2);
                            _unfriendButton.addEventListener(Event.TRIGGERED,deleteFriendEventHandler);
                            _unfriendButton.layoutData = buttonLayData();
                            _buttonContainer.addChild(_unfriendButton);

                            _itemsTohide.push(_unfriendButton);

                            _slidedButton = new SlidingButton();
                            _slidedButton.layoutData = buttonLayData();
                            _buttonContainer.addChild(_slidedButton);
                            _itemsTohide.push(_slidedButton);
                        }else{
                            _unfriendButton.visible = true;
                            _slidedButton.visible = true;
                        }
                        break;
                    case Friend.STATE_PENDING:
                        if (!_approveButton)
                        {
                            var layData : AnchorLayoutData = new AnchorLayoutData();
                            layData.bottom = 1;
                            layData.horizontalCenter = 0;
                            _dateLabel = new Label();
                            _dateLabel.textRendererFactory = NotificationPanelLabelFactory.postDateLabelRenderer;
                            _mainContainer.addChild(_dateLabel);
                            _dateLabel.layoutData= layData;
                            _dateLabel.text = DateParser.getDateAppropriateText(friend.availableDate);
                            _itemsTohide.push(_dateLabel);

                            layData = new AnchorLayoutData();
                            layData.verticalCenter = 0;
                            layData.leftAnchorDisplayObject = _nameLabel;
                            layData.left = 2;
                            _descriptionalLabel = new Label();
                            _descriptionalLabel.textRendererFactory = descriptionTextRenderer;
                            _descriptionalLabel.layoutData = layData;
                            _mainContainer.addChild(_descriptionalLabel);
                            _descriptionalLabel.text = 'wants to be friends with you';
                            _itemsTohide.push(_descriptionalLabel);

                            _rejectButton = ButtonFactory.newButton("decline","DECLINE",0x58595B);
                            _rejectButton.layoutData = buttonLayData();
                            _rejectButton.addEventListener(Event.TRIGGERED,rejectFriendRequestEventHandler);
                            _buttonContainer.addChild(_rejectButton);
                            _itemsTohide.push(_rejectButton);

                            _approveButton = ButtonFactory.newButton("accept","ACCEPT",0xFFFFFF);
                            _approveButton.addEventListener(Event.TRIGGERED,confirmFriendRequestEventHandler);
                            _approveButton.layoutData = buttonLayData();
                            (_approveButton.layoutData as AnchorLayoutData).rightAnchorDisplayObject = _rejectButton;
                            _buttonContainer.addChild(_approveButton);
                            _itemsTohide.push(_approveButton);
                        }else{
                            _approveButton.visible = true;
                            _rejectButton.visible = true;
                            _dateLabel.visible = true;
                            _dateLabel.text = DateParser.getDateAppropriateText(friend.availableDate);
                        }
                        break;

                    case Friend.STATE_REQUEST_SENT:
                        if (!_undoFriendButton)
                        {
                            _undoFriendButton = ButtonFactory.newButton("undo_friend","undo",0xF8F8F8);
                            _undoFriendButton.addEventListener(Event.TRIGGERED,undoFriendRequestEventHandler);
                            _undoFriendButton.layoutData = buttonLayData();
                            (_undoFriendButton.layoutData as AnchorLayoutData).verticalCenter = -3;
                            _buttonContainer.addChild(_undoFriendButton);
                            _itemsTohide.push(_undoFriendButton);
                            _undoFriendButton.validate();

                            _requestLabel = new MultilingualLabel();
                            _requestLabel.text = 'request sent';
                            _requestLabel.textRendererFactory = detailsTextRenderer;
                            _requestLabel.layoutData = buttonLayData();
                            (_requestLabel.layoutData as AnchorLayoutData).topAnchorDisplayObject = _undoFriendButton;
                            (_requestLabel.layoutData as AnchorLayoutData).top = 0;
                            (_requestLabel.layoutData as AnchorLayoutData).horizontalCenter = 0;
                            _buttonContainer.addChild(_requestLabel);
                            _itemsTohide.push(_requestLabel);
                        }else{
                            _undoFriendButton.visible = true;
                            _requestLabel.visible = true;
                        }
                        break
                }
            }
		}

        private function onFriendSelectHandler(event:TouchEvent) : void
        {
            var touch : Touch = event.touches[0] as Touch;

            if (touch.phase == TouchPhase.ENDED && !this.owner.isScrolling)
            {
                Constants.selectedUser = friend.user.copy();

                eventDispatcher.dispatchEventWith(StackedScreen.GARAGE_SELECTED);
            }
        }

		public override function dispose() : void
		{
            if (_undoFriendButton){
                _undoFriendButton.removeEventListener(Event.TRIGGERED, undoFriendRequestEventHandler);
                _undoFriendButton.removeFromParent(true);
                _requestLabel.removeFromParent(true);
            }

            if (_addFriendButton){
                _addFriendButton.removeEventListener(Event.TRIGGERED, sendFriendRequestEventHandler);
                _addFriendButton.removeFromParent(true);
            }

            if (_approveButton){
                _approveButton.removeEventListener(Event.TRIGGERED, confirmFriendRequestEventHandler);
                _approveButton.removeFromParent(true);
            }

            if (_rejectButton){
                _rejectButton.removeEventListener(Event.TRIGGERED, rejectFriendRequestEventHandler);
                _rejectButton.removeFromParent(true);
            }

            if (_slidedButton){
                _unfriendButton.removeEventListener(Event.TRIGGERED, deleteFriendEventHandler);
                _unfriendButton.removeFromParent(true);
                _slidedButton.removeFromParent(true);
            }

            if(_userAvatar)
            {
                _userAvatar.removeFromParent(true);
                _userAvatar = null;
            }

            if(_avatarContainer){
                _avatarContainer.removeFromParent(true);
                _avatarContainer.removeEventListener(TouchEvent.TOUCH, onFriendSelectHandler)
                _avatarContainer = null;
            }


            if (verSpacer) {
                removeChild(verSpacer);
            }

            if (_nameLabel){
                _nameLabel.removeEventListener(TouchEvent.TOUCH, onFriendSelectHandler)
                _nameLabel.removeFromParent(true);
            }

            super.dispose();
        }

        private function sendFriendRequestEventHandler(event:Event) : void {
            Friend.PROXY.add_friend(friend);
        }

        private function deleteFriendEventHandler(event:Event):void {
            Friend.PROXY.delete_friend(friend);
        }

        private function undoFriendRequestEventHandler(event:Event) : void {
            Friend.PROXY.undo_friend(friend);
        }

        private function confirmFriendRequestEventHandler(event:Event) : void {
            Friend.PROXY.approveRequest(friend);
        }

        private function rejectFriendRequestEventHandler(event:Event) : void {
            Friend.PROXY.rejectRequest(friend);
        }

		protected override function commitData() : void
		{
            if(_lastFriendData){
                _lastFriendData.removeHolder(this);
            }
            _lastFriendData = friend;
            friend.holder = this;
            //super.commitData();
            updateItemsVisibility();
		}

        public function updateFriendState(state:String):void {
            updateItemsVisibility();
        }
    }
}