package com.despark.tuning3d.view.components.tunings.editing
{
import feathers.controls.Button;
import feathers.core.FeathersControl;

import flash.geom.Point;

import starling.display.Image;
import starling.display.Quad;
import starling.events.Touch;
import starling.events.TouchEvent;
import starling.events.TouchPhase;
import starling.textures.Texture;
import starling.utils.AssetManager;

public class TuningSuspensionControl extends FeathersControl
	{
		public static const SUSPENSION_CHANGED : String = "SuspensionChanged";
		
		public var MAX_HEIGHT : int = 205-15;
		public var MIN_HEIGHT : int = 0;

		public var assetManager : AssetManager;
		
		private var _maxSuspension : int;
		private var _minSuspension : int;
		
		private var _normalMin:int;
		private var _normalMax:int;
		
		private var backgroundImage:Image;
		private var handlerImage:Image;

		private var _value:Number=0;
		
		private var quad:Quad;

		public function TuningSuspensionControl()
		{
			super();
			
			addEventListener(TouchEvent.TOUCH, touchHandler);
			setMinMaxSuspension(15, -15);
			
			
		}
		
		public function setMinMaxSuspension(min:int, max:int):void{
			
			_minSuspension=max;
			_maxSuspension=min;
			
			_normalMax = _maxSuspension;
			_normalMin = _minSuspension;

			invalidate("value");
		}
		
		public function get minSuspension() : int{
			
			return _minSuspension;
		}
		

		public function get maxSuspension() : int
		{
			return _maxSuspension;
		}
		
		override protected function initialize() : void
		{
			super.initialize();
			
			quad = new Quad(1, 1, 0xFF0000);
			quad.alpha = 0.0;
			addChild(quad);
			

			var backgroundTexture : Texture = assetManager.getTexture("a_new_suspension_line");
			backgroundImage = new Image(backgroundTexture);
			//backgroundImage.width = 205;
			backgroundImage.height = 4;
			addChild(backgroundImage);
			
			var handlerTexture:Texture  = assetManager.getTexture("a_new_suspension_handler");
			handlerImage = new Image(handlerTexture);
			handlerImage.width = 15;
			handlerImage.height = 15;
			handlerImage.touchable=false;
			addChild(handlerImage);
		}
		
		override public function dispose():void{
			
			removeEventListener(TouchEvent.TOUCH, touchHandler);
			super.dispose();
			
		}
		

		override protected function draw() : void
		{
			super.draw();
			if(isInvalid(INVALIDATION_FLAG_SIZE))
			{
				backgroundImage.x=0;
				backgroundImage.y=(actualHeight-backgroundImage.height)/2;
				backgroundImage.width = actualWidth;
				backgroundImage.height = 4;
				
				handlerImage.x = posForSuspension(_value);//((((_value)/normal) + (_normalMax/normal))*(backgroundImage.width-handlerImage.width));//((_value/normal)*backgroundImage.width)-(handlerImage.width/2);
				handlerImage.y = (actualHeight-handlerImage.height)/2;
				
				quad.width=actualWidth;
				quad.height=actualHeight;
			}
			
			
			if(isInvalid("value")){

				handlerImage.x =  posForSuspension(_value);
				handlerImage.y =  (actualHeight-handlerImage.height)/2;//-(handlerImage.height-backgroundImage.height)/2;
			}
		}
		
		public function set value(val:Number):void{
			
			_value = val;
			invalidate("value");
		}
		
		public function get value():Number{
			
			return _value;
		}
		
		public function posForSuspension(suspension : int) : Number
		{
			
			var DEST_PX:Number=MAX_HEIGHT-MIN_HEIGHT;
			var dest:Number=(maxSuspension-minSuspension);
			if(dest == 0)
			{
				return (MIN_HEIGHT);
			}
			
			return ((((suspension-minSuspension)/dest)*DEST_PX)+MIN_HEIGHT);
		}
		
		
		public function suspensionForPos(posX : Number) : int
		{
			var DEST_PX:Number=MAX_HEIGHT-MIN_HEIGHT;
			var dest:Number=(maxSuspension-minSuspension);
			return  Math.round((dest-((((posX - MIN_HEIGHT) / DEST_PX)*dest)))-maxSuspension);   // PIXELS_FOR_CHANGE;
		}

        private var _touchBegan:Boolean = false;
		
		private function touchHandler(event : TouchEvent) : void
		{
			var touch : Touch = event.getTouch(this);

			if(maxSuspension-minSuspension == 0)
			{
				return;
			}


			if(!touch)
			{
                _touchBegan = false;
				return;
			}	
            var touchX : Number = this.globalToLocal(new Point(touch.globalX, touch.globalY)).x;
			var dX : Number;
			
			if(touch.phase == TouchPhase.BEGAN)
			{
				_touchBegan = true;
				dX = (touchX);
				if(dX<0) dX=0;
				if(dX>this.actualWidth-this.handlerImage.width) dX = this.actualWidth-this.handlerImage.width;

				value = - this.suspensionForPos(dX);
				
				dispatchEventWith(SUSPENSION_CHANGED, true, {suspension: this.suspensionForPos(dX)});
			}
			else if(touch.phase == TouchPhase.MOVED && _touchBegan)
			{
				dX = (touchX);
				if(dX<0) dX=0;
				if(dX>this.actualWidth-this.handlerImage.width) dX = this.actualWidth-this.handlerImage.width;
					
				value = - this.suspensionForPos(dX);
				dispatchEventWith(SUSPENSION_CHANGED, true, {suspension: this.suspensionForPos(dX)});
			}
			else {
                _touchBegan = false;
			}
		}
	}
}

